// blah.c
//
// Copyright (c) 2001,2002 Andrew McGill and Leading Edge Business Solutions
//
// This file is part of dialserver.
//
// dialserver is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your option)
// any later version.
//
// dialserver is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
// more details.
//
// You should have received a copy of the GNU General Public License along with
// dialserver; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA  02111-1307  USA

#include "common.h"
#include "logic.h"
#include "server.h"
#include "conf.h"
#include "client.h"
#include "blah.h"

#ifdef HAVE_SIGNAL_H
#include <signal.h>
#endif
#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif
#ifdef HAVE_SYS_WAIT_H
#include <sys/wait.h>
#endif

// Take note of terminated child processes (otherwise grimly known as 
// the zombie reaper)
void child_reaper(int status)
{
	pid_t childpid;

	signal (SIGCHLD, child_reaper);  // Linux resets signal handlers ... 
	while ((childpid=waitpid (-1, &status, WNOHANG)) > 0) {
		if (WIFEXITED(status) || WIFSIGNALED(status)) {
			// If it really exited -not just stopped, and you 
			// really cared, here would be a good place to act
			if (child_notify) child_notify(childpid);
		}
	}
}

// Write our PID to a file
void ds_writepid(void)
{
	FILE *f;
	f = fopen(conf.pidfile,"w");
	if (!f) {
		perror("Cannot open PID file");
		exit(1);
	}
	fprintf(f,"%lu\n",(unsigned long)getpid());
	fclose(f);
}

// Remove our lock file
void ds_unwritepid(void)
{
	unlink(conf.pidfile);
}

// Check whether we're already running
void ds_checknotalreadyrunning(void)
{
	unsigned long pid;
	char buffer[16];
	FILE *f;
	int rc;

	f = fopen(conf.pidfile,"r");
	if (!f) return;  // no file? no permissions? no problem!
	buffer[0]=0;
	fgets(buffer,sizeof(buffer),f);
	fclose(f);

	pid = atol(buffer);
	if (!pid) return; // not a number?  no problem.
	rc = kill(pid,0);
	if ((rc!=0 && errno==EPERM) || (rc==0)) {
		fprintf(stderr,"Already running (%lu)\n",(unsigned long)pid);
		exit(1);
	}
	f = fopen(conf.pidfile,"w");
	if (!f) {
		perror("Can't write pid");
		exit(1);  // oops!
	}
	fprintf(f,"%lu\n",(unsigned long)getpid());
	fclose(f);
}

extern char **environ;

// Print out the environement in asciiz terminated strings
void dlr_showenv(void)
{
	int a;
	char c=0;
	for (a=0; environ[a]; a++) {
		fwrite(environ[a], 1, strlen(environ[a])+1, stdout);
	}
	fwrite(&c, 1, 1, stdout);
}

// Main bla
int main(int argc, char *argv[])
{
	// Prevent oodles of defunct zombies
	signal (SIGCHLD, child_reaper); 

	// Special mode of operation - dump environment strings to stdout and exit
	// Used to handle HUP
	if (argc>1 && strcasecmp(argv[1],"showenv")==0) {
		dlr_showenv();
		return 0;
	}

	if (argc>2 && strcasecmp(argv[1],"-c")==0) {
		if (dlr_loadconfig(argv[2], argv[0])) return 1;
		argv = &argv[2];
		argc-=2;
	}
	else {
		if (dlr_loadconfig(CONFIGFILE, argv[0])) return 1;
	}

	// Special mode of operation - single command on command line
	if (argc>1) {
		return cli_docommandline(argc,argv);
	}


	ds_checknotalreadyrunning();
	ds_writepid();
	atexit(ds_unwritepid);

	// Operate as a server, listening on TCP and UNIX sockets
	srv_startserver(conf.tcpport,conf.unixsocket,dlr_server,1);
	return 1;
}

