// log.c -- logging functions (whoo hoo)
//
// Copyright (c) 2001,2002 Andrew McGill and Leading Edge Business Solutions
//
// This file is part of dialserver.
//
// dialserver is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your option)
// any later version.
//
// dialserver is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
// more details.
//
// You should have received a copy of the GNU General Public License along with
// dialserver; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA  02111-1307  USA
#include "common.h"

#ifdef HAVE_TIME_H
#include <time.h>
#endif

#include "util.h"
#include "log.h"
#include "conf.h"

// Returns a pointer to the number of lines wanted from the end
// of the dialer log
char *dlr_logtail(int bytes)
{
	FILE *log;
	char *p, *data;
	long pos;

	data = xmalloc(bytes+1);
	data[bytes] = 0;
	log = fopen(conf.logfile,"r");
	if (!log) {
		snprintf(data,bytes,"%s",strerror(errno));
		free(data);
		return NULL;
	}
	fseek(log, 0L, SEEK_END);
	pos = ftell(log);
	if (pos<bytes) pos=0; else pos=pos-bytes;
	fseek(log, pos, SEEK_SET);
	pos = fread(data,1,bytes,log);
	if (pos<0) {
		if (ferror(log)) {
			snprintf(data,bytes,"%s",strerror(errno));
		}
		else {
			snprintf(data,bytes,"No data");
		}
	}
	else {
		bytes=pos;
		data[bytes]=0;
	}
	fclose(log);
	p = strchr(data,'\n');
	if (p) {
		p++;
		memmove(data, p, bytes-(p-data)+1); // +1? move the null too
	}
	return data;
}


int dlr_openlog(void)
{
	if (stdlog) fclose(stdlog);
	stdlog = fopen(conf.logfile,"a");
	return (stdlog!=NULL);
}

// Logs to conf.logfile and also to stdout
// this (maybe redirect stderr and stdout?)
void dlr_event(char *format, ...)
{
	va_list ap;
	char buffer[24];
	int rc;
	time_t now;
	struct tm *local;
	int a;

	va_start(ap, format);
	time(&now);
	local = localtime(&now);
	// Jul 17 11:34:11
	strftime(buffer, sizeof(buffer), "%b %d %H:%M:%S", local);
	
	for (a=0; a<2; a++) {
		if (stdlog)  {
			rc = fprintf(stdlog,"%s ",buffer);
			vfprintf(stdlog, format, ap);
			fflush(stdlog);
			if (rc>0) break;
		}
		dlr_openlog();
	}
	// Echo it to stdout too -- but not timestamped.  If you don't like
	// this, you can always direct it to /dev/null :)
	vfprintf(stdout, format, ap);
	fflush(stdout);
	va_end(ap);
}

